class Metrics:
    received_packets: int = 0
    sent_packets: int = 0
    forward_key_set: int = 0
    forward_key_del: int = 0
    lost_packet_count: int = 0
    out_of_order_count: int = 0
    delete_unknown_key_count: int = 0
